@@ -19,6 +19,7 @@
 #include "messages.h"
 #include "rockridge.h"
 #include "util.h"
+#include "system_area.h"
 
 #include "libburn/libburn.h"
 
@@ -251,6 +252,7 @@ void write_one_dir_record(Ecma119Image *t, Ecma119Node *node, int file_id,
             : (uint8_t*)node->iso_name;
 
     struct ecma119_dir_record *rec = (struct ecma119_dir_record*)buf;
+    IsoNode *iso;
 
     len_dr = 33 + len_fi + (len_fi % 2 ? 0 : 1);
 
@@ -288,7 +290,14 @@ void write_one_dir_record(Ecma119Image *t, Ecma119Node *node, int file_id,
     rec->len_dr[0] = len_dr + (info != NULL ? info->suf_len : 0);
     iso_bb(rec->block, block, 4);
     iso_bb(rec->length, len, 4);
-    iso_datetime_7(rec->recording_time, t->now, t->always_gmt);
+    if(t->dir_rec_mtime) {
+        iso= node->node;
+        iso_datetime_7(rec->recording_time,
+                       t->replace_timestamps ? t->timestamp : iso->mtime,
+                       t->always_gmt);
+    } else {
+        iso_datetime_7(rec->recording_time, t->now, t->always_gmt);
+    }
     rec->flags[0] = ((node->type == ECMA119_DIR) ? 2 : 0) | (multi_extend ? 0x80 : 0);
     iso_bb(rec->vol_seq_number, 1, 2);
     rec->len_fi[0] = len_fi;
@@ -316,8 +325,8 @@ char *get_relaxed_vol_id(Ecma119Image *t, const char *name)
             return str;
         }
         iso_msg_submit(t->image->id, ISO_FILENAME_WRONG_CHARSET, ret,
-                  "Charset conversion error. Can't convert %s from %s to %s",
-                  name, t->input_charset, t->output_charset);
+                  "Charset conversion error. Cannot convert from %s to %s",
+                  t->input_charset, t->output_charset);
     }
     return strdup(name);
 }
@@ -784,10 +793,14 @@ void *write_function(void *arg)
     target->bytes_written = (off_t) 0;
     target->percent_written = 0;
 
-    /* Write System Area, 16 blocks of zeros (ECMA-119, 6.2.1) */
-    memset(buf, 0, BLOCK_SIZE);
-    for (i = 0; i < 16; ++i) {
-        res = iso_write(target, buf, BLOCK_SIZE);
+    /* Write System Area (ECMA-119, 6.2.1) */
+    {
+        uint8_t sa[16 * BLOCK_SIZE];
+        res = iso_write_system_area(target, sa);
+        if (res < 0) {
+            goto write_error;
+        }
+        res = iso_write(target, sa, 16 * BLOCK_SIZE);
         if (res < 0) {
             goto write_error;
         }
@@ -848,6 +861,7 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
 {
     int ret, i, voldesc_size, nwriters;
     Ecma119Image *target;
+    int el_torito_writer_index = -1;
 
     /* 1. Allocate target and copy opts there */
     target = calloc(1, sizeof(Ecma119Image));
@@ -881,6 +895,8 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
     target->allow_full_ascii = opts->allow_full_ascii;
     target->relaxed_vol_atts = opts->relaxed_vol_atts;
     target->joliet_longer_paths = opts->joliet_longer_paths;
+    target->rrip_version_1_10 = opts->rrip_version_1_10;
+    target->dir_rec_mtime = opts->dir_rec_mtime;
     target->sort_files = opts->sort_files;
 
     target->replace_uid = opts->replace_uid ? 1 : 0;
@@ -906,8 +922,13 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
     target->catalog = src->bootcat;
 
     /* default to locale charset */
+    /* ??? ts Nov 25 2008 :
+       Shouldn't this go to library initialization or even to app ?
+    */
     setlocale(LC_CTYPE, "");
-    target->input_charset = strdup(nl_langinfo(CODESET));
+
+    target->input_charset = strdup(iso_get_local_charset(0));
+
     if (target->input_charset == NULL) {
         iso_image_unref(src);
         free(target);
@@ -967,6 +988,7 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
         if (ret < 0) {
             goto target_cleanup;
         }
+        el_torito_writer_index = target->nwriters - 1;
     }
 
     /* create writer for Joliet structure */
@@ -1013,11 +1035,28 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
      */
     for (i = 0; i < target->nwriters; ++i) {
         IsoImageWriter *writer = target->writers[i];
+
+#define Libisofs_patch_ticket_145 yes
+#ifdef Libisofs_patch_ticket_145
+        /* Delaying boot image patching until new LBA is known */
+        if (i == el_torito_writer_index)
+    continue;
+#endif
         ret = writer->compute_data_blocks(writer);
         if (ret < 0) {
             goto target_cleanup;
         }
     }
+#ifdef Libisofs_patch_ticket_145
+    /* Now perform delayed image patching */
+    if (el_torito_writer_index >= 0) {
+        IsoImageWriter *writer = target->writers[el_torito_writer_index];
+        ret = writer->compute_data_blocks(writer);
+        if (ret < 0) {
+            goto target_cleanup;
+        }
+    }
+#endif /* Libisofs_patch_ticket_145 */
 
     /* create the ring buffer */
     ret = iso_ring_buffer_new(opts->fifo_size, &target->buffer);
@@ -1054,6 +1093,14 @@ int ecma119_image_new(IsoImage *src, IsoWriteOpts *opts, Ecma119Image **img)
             }
         }
 
+        /* write the system area */
+        ret = iso_write_system_area(target, opts->overwrite);
+        if (ret < 0) {
+            iso_msg_debug(target->image->id,
+                          "Error writing system area to overwrite buffer");
+            goto target_cleanup;
+        }
+
         /* skip the first 16 blocks (system area) */
         buf = opts->overwrite + 16 * BLOCK_SIZE;
         voldesc_size *= BLOCK_SIZE;
@@ -1446,6 +1493,24 @@ int iso_write_opts_set_joliet_longer_paths(IsoWriteOpts *opts, int allow)
     return ISO_SUCCESS;
 }
 
+int iso_write_opts_set_rrip_version_1_10(IsoWriteOpts *opts, int oldvers)
+{
+    if (opts == NULL) {
+        return ISO_NULL_POINTER;
+    }
+    opts->rrip_version_1_10 = oldvers ? 1 : 0;
+    return ISO_SUCCESS;
+}
+
+int iso_write_opts_set_dir_rec_mtime(IsoWriteOpts *opts, int allow)
+{
+    if (opts == NULL) {
+        return ISO_NULL_POINTER;
+    }
+    opts->dir_rec_mtime = allow ? 1 : 0;
+    return ISO_SUCCESS;
+}
+
 int iso_write_opts_set_sort_files(IsoWriteOpts *opts, int sort)
 {
     if (opts == NULL) {
