@@ -105,6 +105,22 @@ struct iso_write_opts {
      */
     unsigned int joliet_longer_paths :1;
 
+    /**
+     * Write Rock Ridge info as of specification RRIP-1.10 rather than
+     * RRIP-1.12: signature "RRIP_1991A" rather than "IEEE_1282",
+     *            field PX without file serial number
+     */
+    unsigned int rrip_version_1_10 :1;
+
+    /**
+     * Store as ECMA-119 Directory Record timestamp the mtime of the source
+     * rather than the image creation time. (The ECMA-119 prescription seems
+     * to expect that we do have a creation timestamp with the source.
+     * mkisofs writes mtimes and the result seems more suitable if mounted
+     * without Rock Ridge support.)
+     */
+    unsigned int dir_rec_mtime :1;
+
     /** If files should be sorted based on their weight. */
     unsigned int sort_files :1;
 
@@ -246,6 +262,12 @@ struct ecma119_image
     /** Allow paths on Joliet tree to be larger than 240 bytes */
     unsigned int joliet_longer_paths :1;
 
+    /** Write old fashioned RRIP-1.10 rather than RRIP-1.12 */
+    unsigned int rrip_version_1_10 :1;
+
+    /* Store in ECMA-119 timestamp mtime of source */
+    unsigned int dir_rec_mtime :1;
+
     /*
      * Mode replace. If one of these flags is set, the correspodent values are
      * replaced with values below.
