@@ -944,8 +944,8 @@ char *get_name(_ImageFsData *fsdata, const char *str, size_t len)
             return name;
         } else {
             ret = iso_msg_submit(fsdata->msgid, ISO_FILENAME_WRONG_CHARSET, ret,
-                "Charset conversion error. Can't convert %s from %s to %s",
-                str, fsdata->input_charset, fsdata->local_charset);
+                "Charset conversion error. Cannot convert from %s to %s",
+                fsdata->input_charset, fsdata->local_charset);
             if (ret < 0) {
                 return NULL; /* aborted */
             }
@@ -1012,8 +1012,8 @@ int iso_file_source_new_ifs(IsoImageFilesystem *fs, IsoFileSource *parent,
     if (record->file_unit_size[0] || record->interleave_gap_size[0]) {
         iso_msg_submit(fsdata->msgid, ISO_UNSUPPORTED_ECMA119, 0,
               "Unsupported image. This image has at least one file recorded "
-              "in interleaved mode. We don't support this mode, as we think "
-              "it's not used. If you're reading this, then we're wrong :) "
+              "in interleaved mode. We do not support this mode, as we think "
+              "it is not used. If you are reading this, then we are wrong :) "
               "Please contact libisofs developers, so we can fix this.");
         return ISO_UNSUPPORTED_ECMA119;
     }
@@ -1040,7 +1040,7 @@ int iso_file_source_new_ifs(IsoImageFilesystem *fs, IsoFileSource *parent,
         char* new_name = get_name(fsdata, (char*)record->file_id, record->len_fi[0]);
         if (new_name == NULL) {
             iso_msg_submit(fsdata->msgid, ISO_WRONG_ECMA119, 0,
-                          "Can't retrieve file name");
+                          "Cannot retrieve file name");
             return ISO_WRONG_ECMA119;
         }
         if (strcmp(new_name, data->name)) {
@@ -1080,7 +1080,7 @@ int iso_file_source_new_ifs(IsoImageFilesystem *fs, IsoFileSource *parent,
             ifsdata->name = get_name(fsdata, (char*)record->file_id, record->len_fi[0]);
             if (ifsdata->name == NULL) {
                 iso_msg_submit(fsdata->msgid, ISO_WRONG_ECMA119, 0,
-                              "Can't retrieve file name");
+                              "Cannot retrieve file name");
                 ret = ISO_WRONG_ECMA119;
                 goto ifs_cleanup;
             }
@@ -1206,7 +1206,10 @@ int iso_file_source_new_ifs(IsoImageFilesystem *fs, IsoFileSource *parent,
                                      "Sparse files not supported.");
                 break;
             } else if (SUSP_SIG(sue, 'R', 'R')) {
-                /* TODO I've seen this RR on mkisofs images. what's this? */
+                /* This was an optional flag byte in RRIP 1.09 which told the
+                   reader what other RRIP fields to expect.
+                   mkisofs emits it. We don't.
+                */
                 continue;
             } else if (SUSP_SIG(sue, 'S', 'P')) {
                 /*
@@ -1275,8 +1278,8 @@ int iso_file_source_new_ifs(IsoImageFilesystem *fs, IsoFileSource *parent,
             if (ret < 0) {
                 /* its just a hint message */
                 ret = iso_msg_submit(fsdata->msgid, ISO_FILENAME_WRONG_CHARSET,
-                                 ret, "Charset conversion error. Can't "
-                                 "convert %s from %s to %s", name,
+                                 ret, "Charset conversion error. Cannot "
+                                 "convert from %s to %s",
                                  fsdata->input_charset, fsdata->local_charset);
                 free(newname);
                 if (ret < 0) {
@@ -1297,8 +1300,8 @@ int iso_file_source_new_ifs(IsoImageFilesystem *fs, IsoFileSource *parent,
                           fsdata->local_charset, &newlinkdest);
             if (ret < 0) {
                 ret = iso_msg_submit(fsdata->msgid, ISO_FILENAME_WRONG_CHARSET,
-                                 ret, "Charset conversion error. Can't "
-                                 "convert %s from %s to %s", name,
+                                 ret, "Charset conversion error. Cannot "
+                                 "convert from %s to %s",
                                  fsdata->input_charset, fsdata->local_charset);
                 free(newlinkdest);
                 if (ret < 0) {
@@ -1340,7 +1343,7 @@ int iso_file_source_new_ifs(IsoImageFilesystem *fs, IsoFileSource *parent,
             name = get_name(fsdata, (char*)record->file_id, record->len_fi[0]);
             if (name == NULL) {
                 return iso_msg_submit(fsdata->msgid, ISO_WRONG_ECMA119, 0,
-                              "Can't retrieve file name");
+                              "Cannot retrieve file name");
             }
 
             /* remove trailing version number */
@@ -1385,13 +1388,41 @@ int iso_file_source_new_ifs(IsoImageFilesystem *fs, IsoFileSource *parent,
         return ISO_SUCCESS;
     }
 
+    /* ts Nov 25 2008: TODO
+       This seems not fully consistent with read_rr_PX() which decides
+       by (px->len_sue[0] == 44) whether an inode number is present or not.
+       What if read_rr_PX finds a PX of length 36 in a IEEE_1282 image ?
+       It is illegal but could confuse the image by duplicate inode numbers.
+
+       Regrettably it is not enough to just use single default numbers.
+       If only one number misses in the image, then all would need to be
+       defaulted by the following iso_global_inode code.
+
+       Why do duplicate inode numbers confuse the file lengths, anyway ?
+       (See ticket 144)
+    */
     if (fsdata->rr != RR_EXT_112) {
         /*
          * Only RRIP 1.12 provides valid inode numbers. If not, it is not easy
          * to generate those serial numbers, and we use extend block instead.
          * It BREAKS POSIX SEMANTICS, but its suitable for our needs
          */
+
+#define Libisofs_patch_ticket_144 yes
+#ifdef Libisofs_patch_ticket_144
+        /* Trying to ensure unique inode numbers */
+        {
+            static ino_t iso_global_inode= 0;
+
+            if(iso_global_inode <= 0)
+                iso_global_inode= 1;
+            atts.st_ino = iso_global_inode++;
+        }
+#else
+        /* Ticket 144: This produces duplicate numbers with empty files.
+        */
         atts.st_ino = (ino_t) iso_read_bb(record->block, 4, NULL);
+#endif
         if (fsdata->rr == 0) {
             atts.st_nlink = 1;
         }
@@ -1996,8 +2027,12 @@ int iso_image_filesystem_new(IsoDataSource *src, struct iso_read_opts *opts,
     data->dir_mode = opts->dir_mode & ~S_IFMT;
     data->msgid = msgid;
 
+    /* ??? ts Nov 25 2008 :
+       Shouldn't this go to library initialization or even to app ?
+    */
     setlocale(LC_CTYPE, "");
-    data->local_charset = strdup(nl_langinfo(CODESET));
+
+    data->local_charset = strdup(iso_get_local_charset(0));
     if (data->local_charset == NULL) {
         ret = ISO_OUT_OF_MEM;
         goto fs_cleanup;
